#!/bin/bash
#
# This tool simplifies the complete + partial updates generation by doing
# everything by itself, just call it !
# Author: Quentin Raynaud
#

. $(dirname "$0")/common.sh

# Checking MAR variable and getting out if not set
if test -z "$MAR"; then
    echo "MAR envirnment variable is not set"
    exit 1
fi

# Set workdir and root variables
workdir=`pwd` 
cd $(dirname "$0")
root=`pwd`
cd "$workdir"

# Insuring MAR variable is set using an absolute PATH
cd $(dirname "$MAR")
MAR=`pwd`/mar
cd "$workdir"
echo "MAR: $MAR"

# Doing the same for MAR_DIR
cd "$MAR_DIR"
MAR_DIR=`pwd`
cd "$workdir"
echo "MAR_DIR: $MAR_DIR"

# -----------------------------------------------------------------------------

if test -z "$MAR_DIR"; then
    echo "You HAVE to set MAR_DIR environment variable to specify" \
	"where are the previous mar files to create partial patchs..."
    exit
fi

# Try to download the previous mars
if test -n "$SSH_SERVER" -a -n "$SSH_USERNAME" -a \
    -n "$SSH_COMMAND"; then
    mars=`ssh $SSH_USERNAME@$SSH_SERVER $SSH_COMMAND` 
    i=1
    for file in $mars; do
	scp "$SSH_USERNAME"@"$SSH_SERVER":"$file" "$MAR_DIR/$i.mar"
	i=$(($i + 1))
    done
fi

# For each mar, unwrap and create the partial
files=`ls $MAR_DIR/*.mar 2>/dev/null`;
if test -z "$files"; then
    echo "No mar files in $MAR_DIR... No partial updates are" \
	"going to be built."
fi

for file in $files; do
    # If not file, continue
    if test ! -f "$file"; then
	continue
    fi

    # Delete temp folder
    rm -Rf "$MAR_DIR/tmp/"

    # Create a tmp dir
    mkdir -p "$MAR_DIR/tmp/"

    # Unwrap mar file
    cd "$MAR_DIR/tmp" && \
	"$root"/unwrap_full_update.pl ../$(basename "$file") && \
	cd "$workdir"
    if test ! $? -eq 0; then
	echo "Error unwrapping mar $file" >&2
	exit 1
    fi

    # Retrieve the mar's BuildID
    appini=$MAR_DIR/tmp/application.ini
    if test ! -f $appini; then
	appini=$MAR_DIR/tmp/Contents/MacOS/application.ini
	if test ! -f $appini; then
	    echo "Could not find application.ini" >&2
	    continue
	fi
    fi
    buildID=`grep ^BuildID= $appini | cut -d = -f 2`
    echo "Creating partial mar from build $buildID..."

    # Creating the partial mar
    SRC_BUILD="$MAR_DIR/tmp/" \
	SRC_BUILD_ID="$buildID" \
	DST_BUILD="$2" \
	make partial-patch
done

echo "The following MAR files were created:"
ls "$1"/*.mar

# Delete temp folder
rm -Rf "$MAR_DIR"/tmp

# Do we need to upload mar files?
files=`ls "$1"/*.mar 2>/dev/null`
if test -z "$SSH_SERVER" -o -z "$SSH_USERNAME" -o \
    -z "$SSH_UPLOAD_DIR" -o -z "$files"; then
    echo "No upload..."
    exit
fi

echo "Uploading mar files..."
scp "$1"/*.mar "$SSH_USERNAME"@"$SSH_SERVER":"$SSH_UPLOAD_DIR"/

if test -n "$SSH_UPLOAD_COMMAND"; then
    ssh_call='echo "Executing the remote command for each mar"'
    for upMar in `ls "$1"/*.mar`; do
	upMar=`basename "$upMar"`
	ssh_call="$ssh_call && "`echo "$SSH_UPLOAD_COMMAND" | sed s@'$UPMAR'@"$upMar"@`
    done

    ssh "$SSH_USERNAME"@"$SSH_SERVER" bash -c "$ssh_call"
fi
